PROGRAM almaqc
  !
  !
  !  This code will do a quality check on netCDF files which wish to comply to the ALMA
  !  standard.
  !
  !  alpha version : 25/09/00
  !
  !  Please send any comments or bug reports you might have to Jan Polcher (polcher@lmd.jussieu.fr)
  !
  !
  IMPLICIT NONE
  !
  INCLUDE "netcdf.inc"
  !
  INTEGER :: iread, ier, nb_files
  CHARACTER(LEN=80), ALLOCATABLE :: names(:) 
  LOGICAL :: check=.FALSE.
  !
  CHARACTER(LEN=80) :: conf_file
  INTEGER :: nbvar_max, nbvar
  CHARACTER(LEN=80), ALLOCATABLE :: var_names(:), var_units(:)
  CHARACTER(LEN=1),  ALLOCATABLE :: var_sgnconv(:), var_level(:)
  LOGICAL, ALLOCATABLE :: var_found(:)
  CHARACTER(LEN=250) :: not_recognized
  CHARACTER(LEN=30) :: conv_str
  REAL, ALLOCATABLE :: var_max(:), var_min(:)
  LOGICAL :: sgn_traditional
  !
  INTEGER :: ncid, iff
  !
  INTEGER iargc
  EXTERNAL iargc
  !
  !
  !  Default values
  !
  conf_file = 'almaqc.conf'
  nbvar_max = 200
  !
  !  Don't know how to do this in the subroutine nct_getarg, If you know
  !  please tell me ! (Jan)
  !
  iread = iargc()
  !
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !
  !     Get the list of file to treat
  !     First we allocate some space to store the names.
  !
  ALLOCATE (names(iread), stat=ier)
  IF (ier.NE.0) THEN
     WRITE (*,*) ' Could not allocate names of size ', iread
     STOP 'almaqc'
  END IF
  !
  !    Get input
  !
  CALL almaqc_getarg(iread, nb_files, names, conf_file, nbvar_max, check)
  !
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !
  !   Get the configuration file
  !   We need to allocate the memory to keep this information
  !
  nbvar = 0
  !
  ALLOCATE (var_names(nbvar_max), stat=ier)
  IF (ier.NE.0) THEN
     WRITE (*,*) ' Could not allocate var_names of size ', nbvar_max
     STOP 'almaqc'
  END IF
  !
  ALLOCATE (var_units(nbvar_max), stat=ier)
  IF (ier.NE.0) THEN
     WRITE (*,*) ' Could not allocate var_units of size ', nbvar_max
     STOP 'almaqc'
  END IF
  !
  ALLOCATE (var_sgnconv(nbvar_max), stat=ier)
  IF (ier.NE.0) THEN
     WRITE (*,*) ' Could not allocate var_sgnconv of size ', nbvar_max
     STOP 'almaqc'
  END IF
  !
  ALLOCATE (var_level(nbvar_max), stat=ier)
  IF (ier.NE.0) THEN
     WRITE (*,*) ' Could not allocate var_level of size ', nbvar_max
     STOP 'almaqc'
  END IF
  !
  ALLOCATE (var_max(nbvar_max), stat=ier)
  IF (ier.NE.0) THEN
     WRITE (*,*) ' Could not allocate var_max of size ', nbvar_max
     STOP 'almaqc'
  END IF
  !
  ALLOCATE (var_min(nbvar_max), stat=ier)
  IF (ier.NE.0) THEN
     WRITE (*,*) ' Could not allocate var_min of size ', nbvar_max
     STOP 'almaqc'
  END IF
  !
  ALLOCATE (var_found(nbvar_max), stat=ier)
  IF (ier.NE.0) THEN
     WRITE (*,*) ' Could not allocate var_found of size ', nbvar_max
     STOP 'almaqc'
  END IF
  var_found(:) = .FALSE.
  !
  !
  CALL almaqc_getconf(conf_file, nbvar_max, check, nbvar,&
       & var_names, var_units, var_sgnconv, var_level, var_max, var_min)
  !
  !
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !
  !  Go through all files and check for each variable if it correct.
  !
  DO iff = 1, nb_files
     !
     ier = NF_OPEN(names(iff), NF_NOWRITE, ncid)
     !
     IF ( ier .NE. NF_NOERR ) THEN
        WRITE(*,*) 'Error while opening file ', names(iff)
        WRITE(*,*) NF_STRERROR(ier)
        STOP
     ENDIF
     !
     !  Get the attribute for the sign convention
     !
     conv_str = ' '
     ier = NF_GET_ATT_TEXT(ncid, NF_GLOBAL, 'SurfSgn_convention', conv_str)
     !
     IF ( ier .NE. NF_NOERR ) THEN
        sgn_traditional = .TRUE.
     ELSE
        CALL strlowercase (conv_str)
        IF ( index(conv_str,'tradi') .GE. 1 ) THEN
           sgn_traditional = .TRUE.
        ELSE IF ( index(conv_str,'mathema') .GE. 1 ) THEN
           sgn_traditional = .FALSE.
        ELSE
           WRITE(*,*) 'Unknown sign convention, please check the netCDF file'
           STOP
        ENDIF
     ENDIF
     !
     !  Inform the user of what we found for the sign convention
     !
     IF ( sgn_traditional ) THEN
        WRITE(*,*) 'Using TRADITIONAL sign convention for the quality checks.'
     ELSE
        WRITE(*,*) 'Using the MATHEMATICAL sign convention for the  quality checks.'
     ENDIF
     !
     !  Now we go through the variables and do the work
     !
     CALL almaqc_checkvar(ncid, nbvar, var_names, var_units, var_sgnconv, var_level, &
          &  var_max, var_min, var_found, sgn_traditional, not_recognized, check)
     !
     CALL almaqc_bilan(iff, nb_files, names, nbvar, var_names, var_level, var_found,&
          & not_recognized)
     !
  ENDDO
  !
END PROGRAM almaqc
!
!====================================================================
!
SUBROUTINE almaqc_getarg(argx, nb_files, names, conf_file, nbvar_max, check)
  !
  !     Read the arguments of almaqc.
  !
  IMPLICIT NONE
  !
  !  ARGUMENTS
  !
  INTEGER, INTENT(in) :: argx
  !
  INTEGER, INTENT(out) :: nb_files
  CHARACTER(LEN=80), INTENT(out) :: names(argx)
  CHARACTER(LEN=80), INTENT(out) :: conf_file
  INTEGER, INTENT(OUT) ::  nbvar_max
  !
  LOGICAL, INTENT(OUT) :: check
  !
  !  LOCAL
  !
  INTEGER :: iret, iread
  CHARACTER(LEN=80) :: tmp, tmp_arg
  CHARACTER(LEN=3) :: fmt
  !
  INTEGER getarg
  EXTERNAL getarg
  !
  !     Get the number of arguments
  !
  nb_files = 0
  !
  iread = 1
  !
  DO WHILE (iread .LE. argx)
     !
     iret = getarg(iread,tmp)
     !
     SELECTCASE(tmp)
        !
     CASE('-d')
        !
        WRITE(*,*) 'DEBUG MODE SELECTED'
        check = .TRUE.
        iread = iread + 1
        !
     CASE('-h')
        !
        WRITE(*,*) 'Usage : almaqc [options] file1 [file2 ...]'
        WRITE(*,*) '    -d : Verbose mode'
        WRITE(*,*) '    -h : This output'
        WRITE(*,*) '    -cf name : Name of configuration file'
        WRITE(*,*) '    -vm int : Integer value of maximum of variable to check for'
        STOP
        !
     CASE('-cf')
        !
        iread = iread + 1
        iret = getarg(iread, tmp_arg)
        conf_file = tmp_arg(1:len_trim(tmp_arg))
        iread = iread + 1 
        !
     CASE('-vm')
        !
        iread = iread + 1
        iret = getarg(iread, tmp_arg)
        WRITE(fmt,'("I",I1)') len_trim(tmp_arg)
        READ(tmp_arg, FMT=fmt) nbvar_max
        iread = iread + 1 
        !
     CASE DEFAULT
        IF ( check ) WRITE(*,*) 'almaqc_getarg : CASE default'
        IF ( index(tmp,'-') .NE. 1) THEN
           nb_files = nb_files + 1
           names(nb_files) = tmp
           iread = iread + 1
        ELSE
           WRITE(*,*) "WARNING Unknown option ",tmp
           WRITE(*,*) "For more information : almaqc -h"
        ENDIF
     END SELECT
     !
  ENDDO
  !
  !  Check if we have work to do ?
  !
  IF ( nb_files .EQ. 0 ) THEN
     WRITE(*,*) 'No files provided on which the QC should be performed'
     STOP
  ENDIF
  !
END SUBROUTINE almaqc_getarg
!
!====================================================================
!
SUBROUTINE almaqc_getconf(conf_file, nbvar_max, check, nbvar,&
     & var_names, var_units, var_sgnconv, var_level, var_max, var_min)
  !
  !
  !  This routine should be able to read directly the text version of the
  !  HTML page on quality control found at 
  !  http://www.lmd.jussieu.fr/~polcher/ALMA/qc_values.html
  !
  !  ARGUMENTS
  !
  IMPLICIT NONE
  !
  CHARACTER(LEN=80), INTENT(IN) :: conf_file
  INTEGER, INTENT(IN) :: nbvar_max
  LOGICAL, INTENT(IN) :: check
  !
  INTEGER, INTENT(OUT) :: nbvar
  CHARACTER(LEN=80), INTENT(OUT) :: var_names(nbvar_max), var_units(nbvar_max)
  CHARACTER(LEN=1),  INTENT(OUT) :: var_sgnconv(nbvar_max), var_level(nbvar_max)
  REAL, INTENT(OUT) :: var_max(nbvar_max), var_min(nbvar_max)
  !
  !  LOCAL
  !
  INTEGER :: iret, iret2, iret3
  INTEGER :: readlen, blk1, blk2, blk3, blk4, blk5, pt1, pt2
  CHARACTER(LEN=250) :: readline
  CHARACTER(LEN=3) :: tl, dl
  CHARACTER(LEN=10) :: fmt
  !
  !  Open configuration file
  !
  OPEN(unit=12, file=conf_file, STATUS='OLD', IOSTAT=iret)
  !
  IF ( iret .NE. 0 ) THEN
     WRITE(*,*) 'Could not open file ', conf_file(1:len_trim(conf_file))
     WRITE(*,*) 'Please use option -cf to provide another filename for'
     WRITE(*,*) 'for the configuration file'
     STOP
  ENDIF
  !
  iret = 0
  !
  DO WHILE (iret .EQ. 0)
     !
     READ (12,'(A250)',IOSTAT=iret) readline
     ! Take out all redundant blanks
     CALL cmpblank(readline)
     readline = ADJUSTL(readline)
     !
     !
     !  Find the elements we need in this line. These
     !  are very simple :
     !  - 5 blanks
     !  - a dot in the 3rd and 4th word
     !
     readlen = len_trim(readline)
     blk1 = index(readline(1:readlen),' ')
     blk2 = index(readline(blk1+1:readlen),' ')+blk1
     blk3 = index(readline(blk2+1:readlen),' ')+blk2
     blk4 = index(readline(blk3+1:readlen),' ')+blk3
     blk5 = index(readline(blk4+1:readlen),' ')+blk4
     pt1 = index(readline(blk2+1:blk3),".")
     pt2 = index(readline(blk3+1:blk4),".")
     !
     !
     ! Obviously we will only decode this line if we have found the 5 blanks and the two dots
     !
     IF ( iret .EQ. 0 .AND. blk5 .GT. 0 .AND. pt1*pt2 .GT. 0) THEN
        !
        ! Now take the variable out of that line
        !
        var_names(nbvar+1) = readline(1:blk1)
        var_units(nbvar+1) = readline(blk1+1:blk2)
        CALL restexponent(var_units(nbvar+1))
        ! Compute the format for the numbers as some compilers
        ! do not support wild cards in this case.
        WRITE(tl,'(I3.3)') blk3-blk2-1
        WRITE(dl,'(I3.3)') blk3-blk2-1 - pt1
        fmt = '(f'//tl//'.'//dl//')' 
        READ (readline(blk2+1:blk3),FMT=fmt,IOSTAT=iret2) var_min(nbvar+1)
        WRITE(tl,'(I3.3)') blk4-blk3-1
        WRITE(dl,'(I3.3)') blk4-blk3-1 - pt2
        fmt = '(f'//tl//'.'//dl//')' 
        READ (readline(blk3+1:blk4),FMT=fmt,IOSTAT=iret3)  var_max(nbvar+1)
        var_sgnconv(nbvar+1) = readline(blk4+1:blk5)
        CALL struppercase(var_sgnconv(nbvar+1))
        var_level(nbvar+1) = readline(blk5+1:readlen)
        CALL struppercase(var_level(nbvar+1))
        !
        ! Check if the READ went well
        !
        IF ( iret2 .EQ. 0 .AND. iret3 .EQ. 0) THEN
           nbvar = nbvar + 1
           !
           IF ( nbvar+1 .GT. nbvar_max) THEN
              WRITE(*,*) 'Insufficient memory to store all variables'
              WRITE(*,*) 'Use the option -vm to allow for a larger number'
              WRITE(*,*) 'of variable to be checked'
              STOP
           ENDIF
           !
           IF ( check ) THEN
              WRITE(*,'("-->",A20,A10,f14.4,f14.4," sgn : ",A1," level : ", A1)') &
                   & var_names(nbvar), var_units(nbvar),&
                   & var_min(nbvar), var_max(nbvar), var_sgnconv(nbvar), var_level(nbvar)
           ENDIF
           !
        ELSE 
           WRITE(*,*) 'Could not read the numbers in this line'
           WRITE(*,*) readline(1:readlen)
           STOP
        ENDIF
        !
        !
     ENDIF
     !
  ENDDO
  !
  IF ( check ) THEN
     WRITE(*,*) 'Found information for ', nbvar,' variables.' 
  ENDIF
  !
END SUBROUTINE almaqc_getconf
!
!====================================================================
!
SUBROUTINE almaqc_checkvar(ncid, nbvar, var_names, var_units, var_sgnconv, var_level,&
     & var_max, var_min, var_found, sgn_traditional, not_recognized, check)
  !
  !
  !  Here we check the various variables in the file
  !
  IMPLICIT NONE
  !
  !  ARGUMENTS
  !
  INTEGER, INTENT(IN) :: ncid
  INTEGER, INTENT(IN) :: nbvar
  CHARACTER(LEN=80), INTENT(IN) ::  var_names(nbvar), var_units(nbvar)
  CHARACTER(LEN=1),  INTENT(OUT) :: var_sgnconv(nbvar), var_level(nbvar)
  REAL, INTENT(IN) :: var_max(nbvar), var_min(nbvar)
  LOGICAL, INTENT(INOUT) ::  var_found(nbvar)
  CHARACTER(LEN=250), INTENT(INOUT):: not_recognized
  LOGICAL, INTENT(IN) :: sgn_traditional
  LOGICAL, INTENT(IN) :: check
  !
  !  LOCAL
  !
  INTEGER :: iret, iv, id, ii, is, il
  INTEGER :: var_infile
  CHARACTER(LEN=80) :: name_infile, units_infile
  INTEGER :: dimids(4), totsize, dimsize(4)
  LOGICAL :: found_units, found_missing
  INTEGER :: found_variable
  REAL, ALLOCATABLE :: varval(:)
  REAL :: missing
  REAL :: fmin, fmax
  INTEGER :: varval_sz=0
  !
  INCLUDE "netcdf.inc"
  !
  !
  not_recognized = ' '
  !
  iret = NF_INQ_NVARS(ncid, var_infile)
  !
  !  Loop through all variables in the file
  !
  DO iv = 1, var_infile
     !
     ! Get the name of variable
     !
     iret = NF_INQ_VARNAME(ncid, iv, name_infile)
     !
     ! Get size of the variable
     !
     dimids(:) = -1
     dimsize(:) = 1
     iret = NF_INQ_VARDIMID(ncid, iv, dimids)
     DO id=1,4
        IF ( dimids(id) .GE. 0) THEN
           iret = NF_INQ_DIMLEN(ncid, dimids(id), dimsize(id))
        ENDIF
     ENDDO
     totsize = dimsize(1)*dimsize(2)*dimsize(3)*dimsize(4)
     !
     ! Get the units
     !
     units_infile = ' '
     iret = NF_GET_ATT_TEXT(ncid, iv, 'units', units_infile)
     IF ( iret .NE. NF_NOERR ) THEN
        iret = NF_GET_ATT_TEXT(ncid, iv, 'UNITS', units_infile)
        IF ( iret .NE. NF_NOERR ) THEN
           found_units = .FALSE.
        ELSE
           found_units = .TRUE.
        ENDIF
     ELSE
        found_units = .TRUE.
     ENDIF
     !
     ! Get missing_value 
     !
     iret = NF_GET_ATT_REAL(ncid, iv, 'missing_value', missing)
     IF ( iret .NE. NF_NOERR ) THEN
        iret = NF_GET_ATT_TEXT(ncid, iv, 'MISSING_value', missing)
        IF ( iret .NE. NF_NOERR ) THEN
           found_missing = .FALSE.
        ELSE
           found_missing = .TRUE.
        ENDIF
     ELSE
        found_missing = .TRUE.
     ENDIF
     !
     ! Does this variable exists in the almaqc.conf file and was it
     ! not found before ?
     !
     found_variable = 0
     DO id = 1, nbvar
        IF ( index(var_names(id), name_infile(1:len_trim(name_infile))) .GT. 0 .AND.&
             & len_trim(var_names(id)) .EQ. len_trim(name_infile)) THEN
           IF ( found_variable .EQ. 0 ) THEN
              found_variable = id
           ENDIF
           var_found(id) = .TRUE.
        ENDIF
     ENDDO
     !
     ! If so, we start to work
     !
     IF ( found_variable .GT. 0 ) THEN
        !
        ! Inform user
        !

        IF ( check ) THEN
           WRITE(*,'("Working on :",A15,", ")') name_infile(1:len_trim(name_infile))
           WRITE(*,*) 'Conf file position :', found_variable
           WRITE(*,*) 'Conf file line, Name : ', var_names(found_variable)(1:len_trim(var_names(found_variable)))
           WRITE(*,*) 'Conf file line, Units : ', var_units(found_variable)(1:len_trim(var_units(found_variable)))
           WRITE(*,*) 'Conf file line, Range :', var_min(found_variable), var_max(found_variable)
        ELSE
           WRITE(*,'("Working on :",A15,", ",$)') name_infile(1:len_trim(name_infile))
        ENDIF
        !
        ! Get the memory to store the variable
        !
        IF ( varval_sz .LT. totsize) THEN
           IF (.NOT. ALLOCATED(varval) ) THEN
              ALLOCATE(varval(totsize), stat=iret)
              varval_sz = totsize
           ELSE
              DEALLOCATE(varval)
              ALLOCATE(varval(totsize), stat=iret)
              varval_sz = totsize
           ENDIF
           IF (iret.NE.0) THEN
              WRITE (*,*) ' Could not allocate varval of size ', totsize
              STOP 'almaqc_checkvar'
           ENDIF
        ENDIF
        !
        ! Check units
        !
        IF ( index(units_infile,var_units(found_variable)) .GT. 0 .AND.&
             & len_trim(units_infile) .EQ. len_trim(var_units(found_variable))) THEN
           WRITE(*,'(A17,$)') 'Units are OK, '
        ELSE
           WRITE(*,*) 'Problem on units : expected = ', var_units(found_variable),' found = ',&
                & units_infile(1:len_trim(units_infile)) 
        ENDIF
        !
        ! Read the data from the file
        !
        iret = NF_GET_VAR_REAL(ncid, iv, varval)
        !
        ! All checks are performed in the traditional sign convention. Thus
        ! if the file follows the mathematical convention and the variable is 
        ! sensitive to the convention then we need to change its sign. 
        !
        IF ( (.NOT. sgn_traditional) .AND. var_sgnconv(found_variable) .EQ. 'Y' ) THEN
           DO ii = 1, totsize
              IF ( ABS(varval(ii) - missing) .GT. EPSILON(missing)) THEN 
                 varval(ii) = varval(ii) * (-1.0)
              ENDIF
           ENDDO
        ENDIF
        !
        ! Compute the min and max values of the field.
        !
        IF ( found_missing ) THEN
           IF ( missing .GT. var_max(found_variable)) THEN
              fmin = MINVAL(varval(1:totsize), MASK=varval(1:totsize) .LT. missing)
              fmax = MAXVAL(varval(1:totsize), MASK=varval(1:totsize) .LT. missing)
           ELSE IF (missing .LT. var_min(found_variable)) THEN
              fmin = MINVAL(varval(1:totsize), MASK=varval(1:totsize) .GT. missing)
              fmax = MAXVAL(varval(1:totsize), MASK=varval(1:totsize) .GT. missing)
           ELSE
              WRITE(*,*) 'The missing value for this variable is within the allowed'
              WRITE(*,*) 'range of the variable'
              STOP
           ENDIF
        ELSE
           fmin = MINVAL(varval(1:totsize))
           fmax = MAXVAL(varval(1:totsize))
        ENDIF
        !
        ! Do the check on the range
        !
        IF ( fmin .GE. var_min(found_variable) .AND. fmax .LE. var_max(found_variable) ) THEN
           WRITE(*,*) 'Range is OK.'
        ELSE
           WRITE(*,*) 'Variables out of range, min/max =', fmin, fmax
        ENDIF
        !
     ELSE
        !
        ! In case the variable does not exist in almaqc.conf we add it
        ! to the list
        !
        is = len_trim(not_recognized)
        il = len_trim(name_infile)
        IF ( is .LE. 1 ) THEN
           not_recognized(is+1:is+il) = name_infile(1:il)
        ELSE
           not_recognized(is+1:is+il+2) = ', '//name_infile(1:il)
        ENDIF
     ENDIF
     !
  ENDDO
  !
END SUBROUTINE almaqc_checkvar
!
!====================================================================
!
!
SUBROUTINE almaqc_bilan(iff, nb_files, names, nbvar, var_names, var_level, var_found,&
     & not_recognized)
  !
  ! This routine will display a resume of variables found and which were checked
  !
  IMPLICIT NONE
  !
  ! Arguments
  !
  INTEGER, INTENT(IN) :: iff, nb_files, nbvar
  CHARACTER(LEN=80), INTENT(IN) :: names(nb_files)
  CHARACTER(LEN=80), INTENT(IN) :: var_names(nbvar)
  CHARACTER(LEN=1), INTENT(IN) :: var_level(nbvar)
  LOGICAL, INTENT(IN) :: var_found(nbvar)
  CHARACTER(LEN=250), INTENT(IN) :: not_recognized
  !
  ! LOCAL
  !
  INTEGER :: iv, is, il, ol, rl, of, rf, ff
  CHARACTER(LEN=250) :: varlist
  !
  !
  WRITE(*,*) 'Resume of variables in file ', names(iff)(1:len_trim(names(iff))),' :'
  !
  IF ( len_trim(not_recognized) .GT. 1) THEN
     WRITE(*,*) 'The variables which were not recognized and thus not checked are'
     WRITE(*,*) not_recognized(1:len_trim(not_recognized))
  ENDIF
  !
  varlist = ' '
  ol = 0
  rl = 0
  of = 0
  rf = 0
  ff = 0
  !
  DO iv=1,nbvar
     !
     ! The Mandatory variables
     !
     IF ( var_level(iv) .EQ. 'M' .AND. (.NOT. var_found(iv)) ) THEN
        is = len_trim(varlist)
        il = len_trim(var_names(iv))
        IF ( is .LE. 1 ) THEN
           varlist(is+1:is+il) = var_names(iv)(1:il)
        ELSE
           varlist(is+1:is+il+2) = ', '//var_names(iv)(1:il)
        ENDIF
     ENDIF
     !
     ! The recommended variables
     !
     IF ( var_level(iv) .EQ. 'R' ) THEN
        rl = rl + 1
        IF ( var_found(iv) ) THEN
           rf = rf + 1
        ENDIF
     ENDIF
     !
     ! The optional variables
     !
     IF ( var_level(iv) .EQ. 'O' ) THEN
        ol = ol + 1
        IF ( var_found(iv) ) THEN
           of = of + 1
        ENDIF
     ENDIF
     !
     ! The forcing variables
     !
     IF ( var_level(iv) .EQ. 'F' ) THEN
        IF ( var_found(iv) ) THEN
           ff = ff + 1
        ENDIF
     ENDIF
  ENDDO
  !
  IF ( ff .GE. 9 ) THEN
     WRITE(*,*) 'A forcing file was checked. No further comments.'
  ELSE
     IF ( len_trim(varlist) .GT. 1) THEN
        WRITE(*,*) 'The following MANDATORY variables are missing :'
        WRITE(*,*) varlist(1:len_trim(varlist))
     ENDIF
     IF ( rf .GT. 0) THEN
        WRITE(*,'(F5.1,"% of the recommended variables were found")') (rf*100.)/rl
     ELSE
        WRITE(*,*) 'None of the recommended variables were found'
     ENDIF
     IF ( of .GT. 0) THEN
        WRITE(*,'(F5.1,"% of the optional variables were found")') (of*100.)/ol
     ELSE
        WRITE(*,*) 'None of the optional variables were found'
     ENDIF
  ENDIF
  !
END SUBROUTINE almaqc_bilan
!
!====================================================================
!
!
SUBROUTINE cmpblank (str)
  !---------------------------------------------------------------------
  !-
  IMPLICIT NONE
  !
  !---------------------------------------------------------------------
  CHARACTER(LEN=*),INTENT(inout) :: str
  !-
  INTEGER :: lcc,ipb
  !---------------------------------------------------------------------
  lcc = LEN_TRIM(str)
  ipb = 1
  DO
     IF (ipb >= lcc)   EXIT
     IF (str(ipb:ipb+1) == '  ') THEN
        str(ipb+1:) = str(ipb+2:lcc)
        lcc = lcc-1
     ELSE
        ipb = ipb+1
     ENDIF
  ENDDO
END SUBROUTINE cmpblank
!
!====================================================================
!
!
SUBROUTINE strlowercase (str)
  !---------------------------------------------------------------------
  !- Converts a string into lowercase
  !---------------------------------------------------------------------
  IMPLICIT NONE
  !-
  CHARACTER(LEN=*) :: str
  !-
  INTEGER :: i,ic
  !---------------------------------------------------------------------
  DO i=1,LEN_TRIM(str)
     ic = IACHAR(str(i:i))
     IF ( (ic >= 65) .AND. (ic <= 90) )   str(i:i) = ACHAR(ic+32)
  ENDDO
  !-----------------------------
END SUBROUTINE strlowercase
!
!====================================================================
!
!
SUBROUTINE struppercase (str)
  !---------------------------------------------------------------------
  !- Converts a string into uppercase
  !---------------------------------------------------------------------
  IMPLICIT NONE
  !-
  CHARACTER(LEN=*) :: str
  !-
  INTEGER :: i,ic
  !---------------------------------------------------------------------
  DO i=1,LEN_TRIM(str)
     ic = IACHAR(str(i:i))
     IF ( (ic >= 97) .AND. (ic <= 122) )   str(i:i) = ACHAR(ic-32)
  ENDDO
  !-----------------------------
END SUBROUTINE struppercase
!
!====================================================================
!
!
SUBROUTINE restexponent(str)
  !
  !- In some cases we have lost the '^' which indicates an exposnent'
  !- To put it back we will look for '2' and '3' and if they are not preceeded
  !- by '^' we put it back in
  !
  IMPLICIT NONE
  !-
  CHARACTER(LEN=*) :: str
  !-
  INTEGER :: il, il2
  !---------------------------------------------------------------------
  !
  IF ( index(str,'2') .GT. 0 ) THEN
     il = index(str,'2')
  ELSE
     il = index(str,'3')
  ENDIF
  !
  DO WHILE ( il .GT. 0 .AND. str(il-1:il-1) .NE. '^' )
     !
     IF ( str(il-1:il-1) .NE. '^' ) THEN
        str = str(1:il-1)//'^'//str(il:len_trim(str))
     ENDIF
     !
     IF ( index(str(il+2:len_trim(str)),'2') .GT. 0 ) THEN
        il2 = index(str(il+2:len_trim(str)),'2')
        il = il + il2 + 1
     ELSE
        il2 = index(str(il+2:len_trim(str)),'3')
        il = il + il2 + 1
     ENDIF
     !
  ENDDO
  !
END SUBROUTINE restexponent
